@@ -1,9 +1,10 @@
 package App::Wubot::Plugin::Facebook;
 use Moose;
 
-our $VERSION = '0.3.1'; # VERSION
+our $VERSION = '0.3.2'; # VERSION
 
 use HTML::TokeParser::Simple;
+use HTML::Strip;
 
 use App::Wubot::Logger;
 use App::Wubot::Util::WebFetcher;
@@ -19,6 +20,14 @@ has 'fetcher' => ( is  => 'ro',
                    },
                );
 
+has 'htmlstrip' => ( is => 'ro',
+                     isa => 'HTML::Strip',
+                     lazy => 1,
+                     default => sub {
+                         return HTML::Strip->new();
+                     },
+                 );
+
 sub check {
     my ( $self, $inputs ) = @_;
 
@@ -87,15 +96,15 @@ sub check {
                     $self->logger->warn( "WARNING: no subject found!" );
                     next TOKEN;
                 }
-                $subject =~ s|\s*\<wbr\/\>\s*||sg;
-
-                $subject = HTML::Entities::decode( $subject );
+                $subject = $self->htmlstrip->parse( $subject );
 
                 if ( $username ) {
                     $username = HTML::Entities::decode( $username );
                 }
 
                 push @messages, { subject  => $subject,
+                                  title    => $subject,
+                                  body     => $subject,
                                   username => $username,
                                   link     => $config->{url},
                               };
@@ -118,11 +127,8 @@ sub check {
                     $subject .= $token->as_is;
                 }
 
-                $subject =~ s|\s*\<wbr\/\>\s*||sg;
-                $subject =~ s|\<div.*?\>||g;
-                $subject =~ s|\<span.*?\>||g;
+                $subject = $self->htmlstrip->parse( $subject );
 
-                $subject = HTML::Entities::decode( $subject );
                 $username = HTML::Entities::decode( $username );
 
                 push @messages, { subject  => $subject,
@@ -178,7 +184,7 @@ App::Wubot::Plugin::Facebook - scrape facebook wall
 
 =head1 VERSION
 
-version 0.3.1
+version 0.3.2
 
 =head1 SYNOPSIS
 
