@@ -1,7 +1,13 @@
 package App::Wubot::Reactor::User;
 use Moose;
 
-our $VERSION = '0.3.1'; # VERSION
+our $VERSION = '0.3.2'; # VERSION
+
+# todo
+#  - App::Wubot::Util::User
+#  - configure class for user in userdb config
+#  - other classes could use other contact back-ends, e.g. emacs contacts
+#  - role for interface, with method to check if file changed
 
 use YAML;
 
@@ -10,10 +16,7 @@ use App::Wubot::Logger;
 has 'userdb'  => ( is => 'ro',
                    isa => 'HashRef',
                    lazy => 1,
-                   default => sub {
-                       my $self = shift;
-                       $self->_read_user_info();
-                   },
+                   default => sub { {} },
                );
 
 has 'directory' => ( is => 'ro',
@@ -32,6 +35,18 @@ has 'logger'  => ( is => 'ro',
                    },
                );
 
+has 'lastupdates'  => ( is => 'ro',
+                        isa => 'HashRef',
+                        lazy => 1,
+                        default => sub { {} },
+                    );
+
+has 'aliases'  => ( is => 'ro',
+                    isa => 'HashRef',
+                    lazy => 1,
+                    default => sub { {} },
+                );
+
 sub react {
     my ( $self, $message, $config ) = @_;
 
@@ -74,18 +89,24 @@ sub react {
         $message->{username_comment} = $2;
     }
 
-    if ( $self->userdb->{ $message->{username} } ) {
+    if ( $message->{username} =~ m/\(http/ ) {
+        $message->{username} =~ m/^(.*?)\s*\((http[^\)]+)/;
+        $message->{username} = $1;
+        $message->{username_comment} = $2;
+    }
+
+    if ( my $userdata = $self->get_user_info( $message->{username} ) ) {
 
         for my $param ( qw( username color image ) ) {
 
             if (    $message->{$param}
                  && ! $message->{"$param\_orig"}
-                 && $message->{$param} ne $self->userdb->{ $message->{username} }->{ $param } ) {
+                 && $message->{$param} ne $userdata->{ $param } ) {
                 $message->{"$param\_orig"} = $message->{$param};
             }
-            if ( $self->userdb->{ $message->{username} }->{ $param } ) {
+            if ( $userdata->{ $param } ) {
                 $self->logger->trace( "Setting $param for $message->{username}" );
-                $message->{$param} = $self->userdb->{ $message->{username} }->{ $param };
+                $message->{$param} = $userdata->{ $param };
             }
         }
     }
@@ -93,7 +114,29 @@ sub react {
     return $message;
 }
 
-sub _read_user_info {
+sub get_user_info {
+    my ( $self, $username ) = @_;
+
+    $username = lc( $username );
+
+    unless ( keys %{ $self->userdb } ) {
+        $self->_read_all_user_info();
+    }
+
+    # look up aliases
+    if ( $self->aliases->{$username} ) {
+        $username = $self->aliases->{$username};
+    }
+
+    return unless $self->userdb->{$username};
+
+    # check for updates to user db
+    $self->_read_userfile( $username );
+
+    return $self->userdb->{$username};
+}
+
+sub _read_all_user_info {
     my ( $self ) = @_;
 
     my $config = {};
@@ -105,29 +148,48 @@ sub _read_user_info {
     while ( defined( my $entry = readdir( $dir_h ) ) ) {
         next unless $entry;
 
-        my $path = join( "/", $directory, $entry );
-        next unless -f $path;
-
         my $user = $entry;
         $user =~ s|.yaml$||g;
 
-        my $user_info = YAML::LoadFile( $path );
-        $user_info->{username} = $user;
-
-        $config->{$user} = $user_info;
-
-        if ( $config->{$user}->{aliases} ) {
-            for my $alias ( keys %{ $config->{$user}->{aliases} } ) {
-
-                $config->{$alias} = $user_info;
-            }
-        }
+        $self->_read_userfile( $user );
     }
     closedir( $dir_h );
 
     return $config;
 }
 
+sub _read_userfile {
+    my ( $self, $username ) = @_;
+
+    my $path = join( "/", $self->directory, "$username.yaml" );
+    return unless -f $path;
+
+    my $mtime = ( stat $path )[9];
+
+    if (    $self->lastupdates->{$username}
+         && $self->lastupdates->{$username} == $mtime ) {
+
+        $self->logger->trace( "User db cache is up to date" );
+        return;
+    }
+
+    my $userdata = YAML::LoadFile( $path );
+    $userdata->{username}   = lc( $username );
+
+    $self->lastupdates->{$username} = $mtime;
+
+    $self->userdb->{$username} = $userdata;
+
+    if ( $userdata->{aliases} ) {
+        for my $alias ( @{ $userdata->{aliases} } ) {
+
+            $alias = lc( $alias );
+            $self->aliases->{$alias} = lc( $username );
+        }
+    }
+
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -141,7 +203,7 @@ App::Wubot::Reactor::User - try to identify user from the 'username' field
 
 =head1 VERSION
 
-version 0.3.1
+version 0.3.2
 
 =head1 SYNOPSIS
 
@@ -188,8 +250,8 @@ Here is an example:
   ---
   color: green
   aliases:
-    lebowski: {}
-    'El Duderino': {}
+    - lebowski
+    - El Duderino
   image: dude.png
 
 If you define a 'color' or an 'image', then any messages that match
@@ -211,6 +273,24 @@ Using the example above, if a message had the username set to
   color: green
   image: dude.png
 
+Each time a message comes through that has a username for which some
+user data is defined, the user's file will be scanned to see if it has
+been updated.  If so, the userdb file will be re-read.
+
+=head1 LIMITATIONS
+
+The file name must be completely lower case, and all usernames and
+aliases will automatically be converted to lower case.  This should be
+fixed in the future.
+
+Adding an alias to a contact will not yet be automatically re-read if
+a message is received from that alias.  In that case it will be
+necessary to restart the reactor to pick up the new alias.  This is
+because it does not know which file to read when it receives a message
+from the new alias.  In the future there should be some mechanism to
+scan the directory occasionally to look for changed files that could
+contain new aliases.
+
 
 =head1 SUBROUTINES/METHODS
 
@@ -220,4 +300,9 @@ Using the example above, if a message had the username set to
 
 The standard reactor plugin react() method.
 
+=item get_user_info( $username )
+
+Given a username or alias, look up the information about the user in
+the userdb.
+
 =back
